/*****************************************************************************
*
* Copyright (c) 2000 - 2015, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

// ************************************************************************* //
//                            avtNektarPPFileFormat.h                        //
// ************************************************************************* //

#ifndef AVT_NektarPP_FILE_FORMAT_H
#define AVT_NektarPP_FILE_FORMAT_H

#include <avtMTSDFileFormat.h>

#include <vtkUnstructuredGrid.h>

#include <SpatialDomains/MeshGraph.h>
#include <MultiRegions/ExpList.h>
#include <SpatialDomains/Geometry.h>

#include <vector>

class vtkDataSet;
class vtkDataArray;
class DBOptionsAttributes;

// ****************************************************************************
//  Class: avtNektarPPFileFormat
//
//  Purpose:
//      Reads in NektarPP files as a plugin to VisIt.
//
//  Programmer: allen -- generated by xml2avt
//  Creation:   Fri Nov 7 13:51:33 PST 2014
//
// ****************************************************************************

class avtNektarPPFileFormat : public avtMTSDFileFormat
{
  public:
                       avtNektarPPFileFormat(const char *, DBOptionsAttributes *);
    virtual           ~avtNektarPPFileFormat() {;};

    virtual void       Initialize();
    virtual void       ActivateTimestep(int ts);

    //
    // This is used to return unconvention data -- ranging from material
    // information to information about block connectivity.
    //
    // virtual void      *GetAuxiliaryData(const char *var, int timestep, 
    //                                     const char *type, void *args, 
    //                                     DestructorFunction &);
    //

    //
    // If you know the times and cycle numbers, overload this function.
    // Otherwise, VisIt will make up some reasonable ones for you.
    //
    virtual void        GetCycles(std::vector<int> &cycles);
    virtual void        GetTimes(std::vector<double> &times);
    //

    virtual int            GetNTimesteps(void);

    virtual const char    *GetType(void)   { return "NektarPP"; };
    virtual void           FreeUpResources(void); 

    virtual vtkDataSet    *GetMesh(int, const char *);
    virtual vtkDataArray  *GetVar(int, const char *);
    virtual vtkDataArray  *GetVectorVar(int, const char *);

  protected:
    // DATA MEMBERS
    std::vector<int>    m_cycles;
    std::vector<double> m_times;

    std::string m_meshFile;

    std::string vectorVarComponents[3];

    int m_nElements;
    std::vector<std::string> m_scalarVarNames;
    std::vector<std::string> m_vectorVarCompNames;

    virtual void           PopulateDatabaseMetaData(avtDatabaseMetaData *, int);

    vtkDataSet *refinedDataSet;

    std::string GetNektarFileAsXMLString( std::string var );

    double GetNektarVar( const Nektar::Array<OneD, NekDouble> &coords,
                         const int nt_el,
                         const int index ) const;
  
    void GetNektarVectorVar( const Nektar::Array<OneD, NekDouble> &coords,
                             const int nt_el,
                             double *vec ) const;

    Nektar::MultiRegions::ExpListSharedPtr nektar_field[3];
};

#endif
